<?php

namespace Drupal\mcapi_signatures;

use Drupal\mcapi\Event\McapiEvents;
use Drupal\mcapi\Event\TransactionAssembleEvent;
use Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * A late subscriber to add signature slots to a transaction while building it.
 */
class TransactionSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() : array{
    return [
      // Must run AFTER any child transactions have been added.
      McapiEvents::ASSEMBLE => ['requireSignatures', -10],
    ];
  }

  /**
   * This is an example for now, but it might work with rules later on.
   *
   * Use $event->getTransaction() to get the parent transaction.
   * Then use $event->addChild($childTransaction)
   *
   * @param TransactionAssembleEvent $event
   *   $event->getSubject() gives the transaction.
   * @param string $eventName
   *   Which is always mcapi_transaction.assemble.
   * @param ContainerAwareEventDispatcher $container
   *   The container.
   */
  public function requireSignatures(TransactionAssembleEvent $event, $eventName, ContainerAwareEventDispatcher $container) {
    $transaction = $event->getSubject();
    if ($transaction->isNew() and !$transaction->isChild) {
      $t_type = $transaction->type->entity;
      if ($relatives = $t_type->getThirdPartySetting('mcapi_signatures', 'relations')) {
        // Signed by the current user
        mcapi_signature_service($transaction)->addSignatures()->sign();
        $transaction->state->setValue('pending');
      }
    }
  }

}
