<?php

namespace Drupal\mcapi\EventSubscriber;

use Drupal\mcapi\Event\McapiEvents;
use Drupal\mcapi\Event\TransactionSaveEvent;
use Drupal\mcapi\Event\TransactionAssembleEvent;
use Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Example subscriber showing the two McapiEvents.
 * @note this class is not listed in mcapi.services.yml
 */
class TransactionSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() : array{
    return [
      McapiEvents::ASSEMBLE => ['makeChildren'],
      McapiEvents::ACTION => ['onTransactionAction']
    ];
  }

  /**
   * This is an example for now, but it might work with rules later on.
   *
   * Use $event->getTransaction() to get the parent transaction.
   * Then use $event->addChild($childTransaction)
   *
   * @param TransactionAssembleEvent $event
   *   $event->getSubject() gives the transaction.
   * @param string $eventName
   *   Which is always mcapi_transaction.assemble.
   * @param ContainerAwareEventDispatcher $container
   *   The container.
   */
  public function makeChildren(TransactionAssembleEvent $event, $eventName, ContainerAwareEventDispatcher $container) {
    $original_transaction = $event->getSubject();
    $config = \Drupal::config('system.site');
    $event->addChildTransaction([
      'payer' => $original_transaction->payer->target_id,
      'payee' => 1,
      'quantity' => 2,
      'description' => t('Payer pays fee to @site', ['@site' => $config->get('name')]),
      'type' => 'inherit'
    ]);
  }

  /**
   * Acts on a transaction and returns a render array in $events->output.
   *
   * @param TransactionSaveEvent $event
   *   $events->getArguments() yields form_values, old_state and operation name,
   *    or action. $events->getSubject() gives the transaction.
   * @param string $eventName
   *   The machine name of the event.
   * @param ContainerAwareEventDispatcher $container
   *   The container.
   */
  public function onTransactionAction(TransactionSaveEvent $event, $eventName, ContainerAwareEventDispatcher $container) {
    $event->setMessage("TransactionSubscriber responded to $eventName");
  }

}
